@@ -1,5 +1,5 @@
 package Crypt::HSM;
-$Crypt::HSM::VERSION = '0.004';
+$Crypt::HSM::VERSION = '0.005';
 use strict;
 use warnings;
 
@@ -22,7 +22,7 @@ Crypt::HSM - A PKCS11 interface for Perl
 
 =head1 VERSION
 
-version 0.004
+version 0.005
 
 =head1 SYNOPSIS
 
@@ -74,7 +74,7 @@ This array lists properties of the mechanism. It may contain values like C<'encr
 
 =head2 open_session($slot, $flags = [])
 
-This opens a session to C<$slot>. C<$flag> is an optional array that may currenlt contain the value C<'rw-session'> to enable writing to the token.
+This opens a session to C<$slot>. C<$flag> is an optional array that may currenlt contain the value C<'rw-session'> to enable writing to the token. This returns a Crypt::HSM::Session object.
 
 =head2 close_all_sessions($slot)
 
